--This class controls all the little snakes
Parent = class('Parent')

function Parent:initialize(speed, scale, growRate, drawMode, delSnakes)
	self.children = {}
	Parent.collisionData = {}
	Parent.collisionData[0] = {}
	Parent.window = {x = math.floor(love.graphics.getWidth() / scale), y =  math.floor(love.graphics.getHeight() / scale)}
	self.food = {x = math.random(0, Parent.window.x - 1), y = math.random(0, Parent.window.y - 1)}	
	self.speed = speed
	if drawMode == 'line' then growRate = 0 end
	self.growRate = growRate
	self.scale = scale
	self.state = 2
	Parent.drawMode = drawMode
	self.delSnakes = delSnakes
end

function Parent:newChild()
	table.insert(self.children, Snake:new(math.random(0, Parent.window.x - 1), math.random(0, Parent.window.y - 1), self.speed, self.growRate))
end

function Parent:killChild(id)
	table.remove(self.children, id)
end

function Parent:pause()
	if self.state == 2 then
		self.state = 1
	elseif self.state == 1 then
		self.state = 2
	end
end

function Parent:updateChildren(dt)
	if self.state == 2 then
		for k, child in ipairs(self.children) do
			--If the destination of the child is different update it to the new one.
			if child.destination.x ~= self.food.x or child.destination.y ~= self.food.y then
				child:setDestination(self.food.x, self.food.y)
			end
			--Give child locations of other children for collision detection
			Parent.collisionData[k] = child:getBody()
			--Move the child
			child:prepareMove(dt)
			--If the child is on food make it grow and place a new food
			if self:onFood(child:getX(), child:getY()) then
				child:grow(self.food.x, self.food.y)
				self:placeFood()
			end
			if child:getState() == 0 and self.delSnakes == 1 then
				self:killChild(k)
			end			
		end
	end
end

--Place a wall
function Parent:placeObstacle(x, y)
	obstacleX = math.floor(x/self.scale)
	obstacleY =  math.floor(y/self.scale)
	for k,v in ipairs(self.collisionData[0]) do
		if v.x == obstacleX and v.y == obstacleY then
			return
		end
	end
	table.insert(self.collisionData[0], {x = obstacleX, y = obstacleY})
end

function Parent:removeObstacle(x, y)
	obstacleX = math.floor(x/self.scale)
	obstacleY =  math.floor(y/self.scale)
	for k,v in ipairs(self.collisionData[0]) do
		if v.x == obstacleX and v.y == obstacleY then
			table.remove(self.collisionData[0], k)
		end
	end	
end

function Parent:draw()
	love.graphics.scale(self.scale)
	self:drawChildren()
	self:drawObstacles()
	self:drawFood()
end

function Parent:drawChildren()
	for k, child in ipairs(self.children) do
		child:draw()
	end
end

function Parent:drawFood()
	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.rectangle("fill", self.food.x, self.food.y, 1, 1)
end

function Parent:drawObstacles()
	love.graphics.setColor(100, 100, 100, 255)
	for k,v in pairs(self.collisionData[0]) do
		love.graphics.rectangle("fill", v.x, v.y, 1, 1)
	end
end

--If X and Y aren't given it's not a user placed food thus we randomize it.
function Parent:placeFood(x, y)
	if x ~= nil and y ~= nil then
		self.food.x = math.floor(x/self.scale)
		self.food.y = math.floor(y/self.scale)
	else
		local newFoodX = math.random(0, Parent.window.x - 1)
		local newFoodY = math.random(0, Parent.window.y - 1)	
		local foodPlaced = false
		while not  foodPlaced do
			local collide = false
			for _,v in pairs(self.collisionData) do
				for _,vv in ipairs(v) do
					if vv.x == newFoodX and vv.y == newFoodY then
						collide = true
						newFoodX = math.random(0, Parent.window.x - 1)
						newFoodY = math.random(0, Parent.window.y - 1)					
					end
				end
			end
			if collide == false then
				foodPlaced = true
			end
		end
		self.food.x = newFoodX
		self.food.y = newFoodY
	end
end

function Parent:onFood(childX, childY)
	if childX == self.food.x and childY == self.food.y then
		return true
	else
		return false
	end
end

function Parent:increaseSpeed(amount)
	if self.speed - amount > 0.00 then
		self.speed = self.speed - amount
		for k, child in ipairs(self.children) do
			child:setSpeed(child:getSpeed() - amount)
		end
	end
end

function Parent:decreaseSpeed(amount)
	self.speed = self.speed + amount
	for k, child in ipairs(self.children) do
		child:setSpeed(child:getSpeed() + amount)
	end
end

function Parent:getSpeed()
	return self.speed
end