--ComputerSnake! Snake for computers!
Snake = class('Snake', Parent)

function Snake:initialize(x, y, speed, growth)
	self.x = x
	self.y = y
	self.speed = speed
	self.body = {{x = self.x, y = self.y}}
	self.length = #self.body
	self.color = {r = math.random(0, 255), g = math.random(0, 255), b = math.random(0, 255)}
	self.direction = 'down'
	self.tdt = 0
	self.state = 1
	self.destination = {}
	self.destination.x, self.destination.y, self.destination.priority = 0, 0, 'down'
	self.growth = growth
end

function Snake:move(dt)
	self.tdt = self.tdt + dt
	if self.tdt > self.speed then
		if self.direction == 'down' then
			self.y = self.y + 1
		elseif self.direction == 'up' then
			self.y = self.y - 1
		elseif self.direction == 'left' then
			self.x = self.x - 1
		elseif self.direction == 'right' then
			self.x = self.x + 1
		end
		self:updateBody()
		self.tdt = 0
	end
end

--This function should've returned true if there's collision
function Snake:checkCollision(futureX, futureY) 
	if futureX < 0 or futureX > Parent.window.x -1  or futureY < 0 or futureY > Parent.window.y - 1 then
		return false
	end
	for k,v in pairs(Parent.collisionData) do
		for kk, vv in ipairs(v) do
			if vv.x == futureX and vv.y == futureY then
				return false
			end
		end
	end
	--If no collision found, we return true
	return true
end

function Snake:setCollisionData(collisionData)
	self.collisionData = collisionData
end

function Snake:getBody()
	return self.body
end

function Snake:getBody()
	return self.body
end

function Snake:setDirection(direction)
	self.direction = direction
end

function Snake:updateBody()
	if Parent.drawMode == 'normal' then
		table.insert(self.body, 1, {x = self:getX(), y = self:getY()})
		table.remove(self.body)
	elseif Parent.drawMode == 'line' then
		table.insert(self.body, 1, {x = self:getX(), y = self:getY()})
	end
end

function Snake:draw()
	love.graphics.setColor(self.color.r, self.color.g, self.color.b, 255)
	for _, v in pairs(self.body) do
		love.graphics.rectangle("fill", v.x, v.y, 1, 1)
	end
end

function Snake:grow(foodX, foodY)
	for i = 1, self.growth do
		table.insert(self.body, {x = foodX, y = foodY})
	end
	self.length = #self.body
end

--This function is pretty ugly right now. 
function Snake:processMove(dt)
	if self.direction == 'up' then
		if self:checkCollision(self:getX(), self:getY() - 1) then
			self.direction = 'up'
		else
			if (self:getX() - self.destination.x) > 0 then
				if self:checkCollision(self:getX() - 1, self:getY()) then
					self.direction = 'left'
				elseif self:checkCollision(self:getX() + 1, self:getY()) then
					self.direction = 'right'
				elseif self:checkCollision(self:getX(), self:getY() + 1) then
					self.direction = 'down'
				else
					self.state = 0
				end
			elseif (self:getX() - self.destination.x) < 0 then
				if self:checkCollision(self:getX() + 1, self:getY()) then
					self.direction = 'right'
				elseif self:checkCollision(self:getX() - 1, self:getY()) then
					self.direction = 'left'
				elseif self:checkCollision(self:getX(), self:getY() + 1) then
					self.direction = 'down'
				else
					self.state = 0
				end
			else
				if math.random(0, 100) > 50 then
					if self:checkCollision(self:getX() - 1, self:getY()) then
						self.direction = 'left'
					elseif self:checkCollision(self:getX() + 1, self:getY()) then
						self.direction = 'right'
					elseif self:checkCollision(self:getX(), self:getY() + 1) then
						self.direction = 'down'
					else
						self.state = 0
					end
				else
					if self:checkCollision(self:getX() + 1, self:getY()) then
						self.direction = 'right'
					elseif self:checkCollision(self:getX() - 1, self:getY()) then
						self.direction = 'left'
					elseif self:checkCollision(self:getX(), self:getY() + 1) then
						self.direction = 'down'
					else
						self.state = 0
					end
				end
			end
		end
	elseif self.direction == 'down' then
		if self:checkCollision(self:getX(), self:getY() + 1) then
			self.direction = 'down'
		else
			if (self:getX() - self.destination.x) > 0 then
				if self:checkCollision(self:getX() - 1, self:getY()) then
					self.direction = 'left'
				elseif self:checkCollision(self:getX() + 1, self:getY()) then
					self.direction = 'right'
				elseif self:checkCollision(self:getX(), self:getY() - 1) then
					self.direction = 'up'
				else
					self.state = 0
				end
			elseif (self:getX() - self.destination.x) < 0 then
				if self:checkCollision(self:getX() + 1, self:getY()) then
					self.direction = 'right'
				elseif self:checkCollision(self:getX() - 1, self:getY()) then
					self.direction = 'left'
				elseif self:checkCollision(self:getX(), self:getY() - 1) then
					self.direction = 'up'
				else
					self.state = 0
				end
			else
				if math.random(0, 100) > 50 then
					if self:checkCollision(self:getX() - 1, self:getY()) then
						self.direction = 'left'
					elseif self:checkCollision(self:getX() + 1, self:getY()) then
						self.direction = 'right'
					elseif self:checkCollision(self:getX(), self:getY() - 1) then
						self.direction = 'up'
					else
						self.state = 0
					end
				else
					if self:checkCollision(self:getX() + 1, self:getY()) then
						self.direction = 'right'
					elseif self:checkCollision(self:getX() - 1, self:getY()) then
						self.direction = 'left'
					elseif self:checkCollision(self:getX(), self:getY() - 1) then
						self.direction = 'up'
					else
						self.state = 0
					end
				end		
			end
		end
	elseif self.direction == 'left' then
		if self:checkCollision(self:getX() - 1, self:getY()) then
			self.direction = 'left'
		else
			if (self:getY() - self.destination.y) > 0 then
				if self:checkCollision(self:getX(), self:getY() - 1) then
					self.direction = 'up'
				elseif self:checkCollision(self:getX(), self:getY() + 1) then
					self.direction = 'down'
				elseif self:checkCollision(self:getX() + 1, self:getY()) then
					self.direction = 'right'
				else
					self.state = 0
				end
			elseif (self:getY() - self.destination.y) < 0 then
				if self:checkCollision(self:getX(), self:getY() + 1) then
					self.direction = 'down'
				elseif self:checkCollision(self:getX(), self:getY() - 1) then
					self.direction = 'up'
				elseif self:checkCollision(self:getX() + 1, self:getY()) then
					self.direction = 'right'
				else
					self.state = 0
				end
			else
				if math.random(0, 100) > 50 then
					if self:checkCollision(self:getX() , self:getY() - 1) then
						self.direction = 'up'
					elseif self:checkCollision(self:getX(), self:getY() + 1) then
						self.direction = 'down'
					elseif self:checkCollision(self:getX() + 1, self:getY()) then
						self.direction = 'right'
					else
						self.state = 0
					end
				else
					if self:checkCollision(self:getX(), self:getY() + 1) then
						self.direction = 'down'
					elseif self:checkCollision(self:getX(), self:getY() - 1) then
						self.direction = 'up'
					elseif self:checkCollision(self:getX() + 1, self:getY()) then
						self.direction = 'right'
					else
						self.state = 0
					end
				end					
			end
		end	
	elseif self.direction == 'right' then
		if self:checkCollision(self:getX() + 1, self:getY()) then
			self.direction = 'right'
		else
			if (self:getY() - self.destination.y) > 0 then
				if self:checkCollision(self:getX(), self:getY() - 1) then
					self.direction = 'up'
				elseif self:checkCollision(self:getX(), self:getY() + 1) then
					self.direction = 'down'
				elseif self:checkCollision(self:getX() - 1, self:getY()) then
					self.direction = 'left'
				else
					self.state = 0
				end
			elseif (self:getY() - self.destination.y) < 0 then
				if self:checkCollision(self:getX(), self:getY() + 1) then
					self.direction = 'down'
				elseif self:checkCollision(self:getX(), self:getY() - 1) then
					self.direction = 'up'
				elseif self:checkCollision(self:getX() - 1, self:getY()) then
					self.direction = 'left'
				else
					self.state = 0
				end
			else
				if math.random(0, 100) > 50 then
					if self:checkCollision(self:getX() , self:getY() - 1) then
						self.direction = 'up'
					elseif self:checkCollision(self:getX(), self:getY() + 1) then
						self.direction = 'down'
					elseif self:checkCollision(self:getX() - 1, self:getY()) then
						self.direction = 'left'
					else
						self.state = 0
					end
				else
					if self:checkCollision(self:getX(), self:getY() + 1) then
						self.direction = 'down'
					elseif self:checkCollision(self:getX(), self:getY() - 1) then
						self.direction = 'up'
					elseif self:checkCollision(self:getX() - 1, self:getY()) then
						self.direction = 'left'
					else
						self.state = 0
					end
				end					
			end
		end		
	end
	self:move(dt)	
end

function Snake:prepareMove(dt)
	if self.state > 0 then
		if self:getX() == self.destination.x then
				self.destination.priority = 'y'		
		elseif self:getY() == self.destination.y then
				self.destination.priority = 'x'
		end		

		if self.destination.priority == 'x'  then
			if (self:getX() - self.destination.x) > 0 then
				self.direction = 'left'
			elseif (self:getX() - self.destination.x) < 0 then
				self.direction = 'right'
			end
			
		elseif self.destination.priority == 'y' then
			if (self:getY() - self.destination.y) > 0 then
				self.direction = 'up'
			elseif (self:getY() - self.destination.y) < 0 then
				self.direction = 'down'
			end
		end
		self:processMove(dt)
	end
end

function Snake:setDestination(destX, destY)
	local tempX, tempY
	self.destination.x = destX
	self.destination.y = destY
	--Find out what direction requires more traveling
	if self:getX() > destX then
		tempX = self:getX() - destX
	elseif  destX > self:getX() then
		tempX = destX
	else
		tempX = 0
	end
	
	if self:getY() > destY then
		tempY = self:getY() - destY
	elseif  destY > self:getY() then
		tempY = destY
	else
		tempY = 0
	end
	
	if tempX > tempY then
		self.destination.priority = 'x'
	elseif tempY > tempX then
		self.destination.priority = 'y'	
	else --if they're both equal we'll just choose a "random" priority
		if math.random(0, 100) > 50 then
			self.destination.priority = 'x'	
		else
			self.destination.priority = 'y'
		end
	end
end

function Snake:getDestination()
	return self.destination.x, self.destination.y, self.destination.priority
end

function Snake:getSpeed()
	return self.speed
end

function Snake:setSpeed(speed)
	self.speed = speed
end

function Snake:getLength()
	return self.length
end

function Snake:getX()
	return math.floor(self.x)
end

function Snake:getY()
	return math.floor(self.y)
end

function Snake:getDestination()
	return self.destination.x, self.destination.y
end

function Snake:getState()
	return self.state
end